import pymysql
import hashlib
import random
import string
import uuid
from datetime import datetime
import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "aiDogProject.settings")
import django
django.setup()
from aiDogApp.models import *
from django.db import transaction

# 响应模板
def get_response_template():
    return {
        "code": "",
        "status": "",
        "msg": "",
        "response_time": datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    }

def generate_random_uid(length=8):
    """生成一个由大写字母和随机数字组成的8位数 uid"""
    characters = string.ascii_uppercase + string.digits
    return ''.join(random.choice(characters) for _ in range(length))


# 用户注册函数
def register_user(mobile_number, password,user_name,UUID,isRegUser):
    response = get_response_template()

    try:
        # 检查手机号是否存在
        if User.objects.filter(mobile=mobile_number).exists():
            response['code'] = 400
            response['status'] = "failed"
            response['msg'] = "手机号已存在"
            return response

        # 开始事务处理
        with transaction.atomic():
            # 生成随机 UID
            uid = generate_random_uid()

            # 哈希加密密码
            hashed_password = hashlib.sha256(password.encode()).hexdigest()

            # 插入用户信息到 User 表
            registration_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            user = User(uid=uid, mobile=mobile_number, password=hashed_password,user_name=user_name, uuid=UUID,isRegUser=isRegUser,registration_time=registration_time)
            user.save()

            # 插入 6 条声纹数据到 VoiceFeatureInfo 表
            # voice_records = []
            # for i in range(7):
            #     command_type = i  # 假定口令类型为数字
            #     voice = VoiceFeatureInfo(uid=uid, command_type=command_type)
            #     voice_records.append(voice)
            #
            # # 批量插入声纹记录
            # VoiceFeatureInfo.objects.bulk_create(voice_records)

            # 新增：插入5条道具数据到 Item 表
            default_items = [
                {'item_id': 'food_00001'},
                {'item_id': 'food_00002'},
                {'item_id': 'food_00003'},
                {'item_id': 'water_00001'},
                {'item_id': 'water_00002'},
                {'item_id': 'toy_00001'},
                {'item_id': 'other_00001'},
                {'item_id': 'other_00002'}
            ]

            # current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            item_list = []
            for item in default_items:
                new_item = UserInventory(
                    owner_id=uid,
                    item_id=item['item_id'],
                    # update_time=current_time
                )
                item_list.append(new_item)

            # 批量插入道具记录
            UserInventory.objects.bulk_create(item_list)

            # 如果一切成功，返回 UID
            response['code'] = 200
            response['status'] = "success"
            response['msg'] = uid
            return response

    except Exception as e:
        # 捕获异常并返回错误信息
        response['code'] = 500
        response['status'] = "error"
        response['msg'] = f"用户注册失败：{str(e)}"
        return response



# 用户email注册函数
def register_user_email(email, password,user_name,UUID,isRegUser):
    response = get_response_template()

    try:
        # 检查手机号是否存在
        if User.objects.filter(email=email).exists():
            response['code'] = 400
            response['status'] = "failed"
            response['msg'] = "This email has been registered!"
            return response

        # 开始事务处理
        with transaction.atomic():
            # 生成随机 UID
            uid = generate_random_uid()

            # 哈希加密密码
            hashed_password = hashlib.sha256(password.encode()).hexdigest()

            # 插入用户信息到 User 表
            registration_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            user = User(uid=uid, email=email, password=hashed_password,user_name=user_name, uuid=UUID,isRegUser=isRegUser,registration_time=registration_time)
            user.save()

            # 插入 6 条声纹数据到 VoiceFeatureInfo 表
            # voice_records = []
            # for i in range(7):
            #     command_type = i  # 假定口令类型为数字
            #     voice = VoiceFeatureInfo(uid=uid, command_type=command_type)
            #     voice_records.append(voice)
            #
            # # 批量插入声纹记录
            # VoiceFeatureInfo.objects.bulk_create(voice_records)

            # 新增：插入默认道具数据到 道具 表
            default_items = [
                {'item_id': 'food_00001'},
                {'item_id': 'food_00002'},
                {'item_id': 'food_00003'},
                {'item_id': 'water_00001'},
                {'item_id': 'water_00002'},
                {'item_id': 'toy_00001'},
                {'item_id': 'other_00001'},
                {'item_id': 'other_00001'}
            ]

            current_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            item_list = []
            for item in default_items:
                new_item = UserInventory(
                    owner_id=uid,
                    item_id=item['item_id'],
                    # update_time=current_time
                )
                item_list.append(new_item)

            # 批量插入道具记录
            UserInventory.objects.bulk_create(item_list)

            # 如果一切成功，返回 UID
            response['code'] = 200
            response['status'] = "success"
            response['msg'] = uid
            return response

    except Exception as e:
        # 捕获异常并返回错误信息
        response['code'] = 500
        response['status'] = "error"
        response['msg'] = f"用户注册失败：{str(e)}"
        return response



# 用户email注册函数
def register_user_email1(email, password,user_name,UUID,isRegUser):
    response = get_response_template()

    try:
        # 检查手机号是否存在
        if User.objects.filter(email=email).exists():
            response['code'] = 400
            response['status'] = "failed"
            response['msg'] = "This email has been registered!"
            return response

        # 开始事务处理
        with transaction.atomic():
            # 生成随机 UID
            uid = generate_random_uid()

            # 哈希加密密码
            hashed_password = hashlib.sha256(password.encode()).hexdigest()

            # 插入用户信息到 User 表
            registration_time = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            user = User(uid=uid, email=email, password=hashed_password,user_name=user_name, uuid=UUID,isRegUser=isRegUser,registration_time=registration_time)
            user.save()

            # 插入 6 条声纹数据到 VoiceFeatureInfo 表
            # voice_records = []
            # for i in range(7):
            #     command_type = i  # 假定口令类型为数字
            #     voice = VoiceFeatureInfo(uid=uid, command_type=command_type)
            #     voice_records.append(voice)
            #
            # # 批量插入声纹记录
            # VoiceFeatureInfo.objects.bulk_create(voice_records)

            # 如果一切成功，返回 UID
            response['code'] = 200
            response['status'] = "success"
            response['msg'] = uid
            return response

    except Exception as e:
        # 捕获异常并返回错误信息
        response['code'] = 500
        response['status'] = "error"
        response['msg'] = f"用户注册失败：{str(e)}"
        return response


# 测试输入用户信息
# mobile_number = '1234567890'
# password = 'ypwd121'
# is_registered = register_user(mobile_number, password)
# if is_registered:
#     print("Registration successful!")
# else:
#     print("Registration failed!")

